#include "PropInfoBase.h"
#include <utility>
#include "XMLFunctions.h"

PropInfoBase::PropInfoBase(
    std::string lightModel, 
    EarthModel earthModel, 
    AtmModel atmModel, 
    const bool sun, 
    const bool moon, 
    const IntegratorCfg& icfg, 
    const int type)
    :
    m_atmModel(std::move(atmModel)),
    m_earthModel(std::move(earthModel)),
    m_lightModel(std::move(lightModel)),
    m_sun(sun),
    m_moon(moon),
    m_type(type),
    m_integrator_cfg(icfg)
{
}

bool PropInfoBase::Load(myXML::Contain* parent)
{
    std::map<std::string, myXML::Contain*> con;
    GetChildren(parent, con);

    std::string type;
    LoadXML(con, "Propagator", type);

    if ((m_type = TypeConvert(type)) == -1) {
        throw (Exp() << "Unsupported model type:" << m_type);
    }

    if (m_type == SGP_PROP || m_type == PROGNOZ_PROP || m_type == KEPLER_PROP) {
        return true;
    }

    bool idloaded;
    LoadXML(con, "Sun", m_sun);
    LoadXML(con, "Moon", m_moon);
    LoadXML(con, "LightModel", m_lightModel);
    LoadXML(con, "EarthModel", m_earthModel);
    LoadXML(con, "AtmModel", m_atmModel, false);
    LoadXML(con, "Id", m_id, false, &idloaded);
    m_integrator_cfg.Load(parent);

    return true;
}

void PropInfoBase::load(const jsonval& parent)
{
    std::string type;
    loadjson(parent, "prop", type);

    if ((m_type = TypeConvert(type)) == -1) {
        throw (Exp() << "Unsupported model type:" << m_type);
    }

    if (m_type == SGP_PROP || m_type == PROGNOZ_PROP || m_type == KEPLER_PROP) {
        return;
    }

    loadjson(parent, "sun", m_sun);
    loadjson(parent, "moon", m_moon);
    loadjson(parent, "light", m_lightModel);
    loadjson(parent, "earth", m_earthModel);
    loadjson(parent, "atm", m_atmModel, false);

    m_integrator_cfg.load(parent);
}

std::string PropInfoBase::GetAsXML() const
{
    std::ostringstream outs;

    PutValue(outs, "Propagator", TypeConvert(m_type));

    if (m_type == SGP_PROP || m_type == PROGNOZ_PROP || m_type == KEPLER_PROP) {
        return outs.str();
    }
    else {
        PutValue(outs, "Sun", m_sun);
        PutValue(outs, "Moon", m_moon);
        PutValue(outs, "LightModel", m_lightModel);
        PutValue(outs, "integratorEps", get_integrator_cfg().getEps());
        PutValue(outs, "integratorStepInit", get_integrator_cfg().getStepInit());
        PutValue(outs, "integratorStepMin", get_integrator_cfg().getStepMin());
        PutValue(outs, "prec_itrf", get_integrator_cfg().getPrecItfr());

        outs << "<EarthModel>";
        PutValue(outs, "Model", m_earthModel.GetModel());
        PutValue(outs, "HarmonicsM", m_earthModel.GetHarmonicsM());
        PutValue(outs, "HarmonicsN", m_earthModel.GetHarmonicsN());
        outs << "</EarthModel>";

        if (m_atmModel.IsUsed()) {
            outs << "<AtmModel>";
            PutValue(outs, "Model", m_atmModel.GetModel());
            PutValue(outs, "KpModel", m_atmModel.GetKpModel());
            outs << "</AtmModel>";
        }
    }

    return outs.str();
}

std::string PropInfoBase::TypeConvert(int value)
{
    if (value == ST_PROP) {
        return "ST";
    }
    else if (value == KEPLER_PROP) {
        return "KEPLER";
    }
    else if (value == PROGNOZ_PROP) {
        return "PROGNOZ";
    }
    else if (value == SGP_PROP) {
        return "SGP";
    }
    else {
        return "";
    }
}

int PropInfoBase::TypeConvert(const std::string& value)
{
    if (value == "ST") {
        return ST_PROP;
    }
    else if (value == "KEPLER") {
        return KEPLER_PROP;
    }
    else if (value == "PROGNOZ") {
        return PROGNOZ_PROP;
    }
    else if (value == "SGP") {
        return SGP_PROP;
    }
    else {
        return -1;
    }
}

void PropInfoBase::ToXml(std::ostream& outs) const
{
    outs << GetAsXML();
}
